/****************************************************************
 AttrRangeTable.H
 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#ifndef INCL_AttrRangeTable_H
#define INCL_AttrRangeTable_H

#include "TrainingSet.H"
#include "tigr++/TigrArray1D.H"
#include "tigr++/TigrSet.H"

/*
  This is an "attribute range table."  It stores, for each
  attribute, the set of all values (or a sampled subset 
  thereof) for that attribute that have been seen in the 
  training set.  This is then used by the procedure which
  builds the tree by considering each attribute and each of
  its values as possible splitting criteria for an internal
  node in the tree.
 */

//---------------------------------------------
struct ValueSet {};
struct DoubleSet : public ValueSet { TigrSet<double> values; };
struct IntSet : public ValueSet { TigrSet<int> values; };
//---------------------------------------------
struct ValueArray 
{
  virtual int getSize()=0;
};
//---------------------------------------------
struct DoubleArray : public ValueArray 
{ 
  TigrArray1D<double> values; 
  DoubleArray(int size) : values(size) {}
  virtual int getSize() {return values.size();}
};
//---------------------------------------------
struct IntArray : public ValueArray 
{ 
  TigrArray1D<int> values; 
  IntArray(int size) : values(size) {}
  virtual int getSize() {return values.size();}
};
//---------------------------------------------
class AttrRangeTable
{
  TrainingSet &trainingSet;
  Schema &schema;
  TigrArray1D<ValueSet*> sets;     // one set per attribute
  TigrArray1D<ValueArray*> arrays; // one array per attribute
  int granularity;

  void computeTable();
  void computeSets();
  void computeArrays();
  void initSets();
  void compressContinuous();
  int aveNumDiscreteVals();
  DoubleArray *compressContinuous(DoubleArray &,int size,
				    int newSize);
public:
  AttrRangeTable(TrainingSet &,int granularity);
  Schema &getSchema();
  TigrArray1D<ValueArray*> &getArrays();
};


#endif
