/****************************************************************
 Classifier.C
 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "Classifier.H"
#include <iostream>


Classifier::Classifier(DecisionTree tree,Schema &schema)
  : tree(tree), schema(schema)
{
  // ctor
}



int Classifier::classify(Case &c)
{
  TreeNode *currentNode=tree.getRoot();
  while(true)
    switch(currentNode->getType())
      {
      case LEAF_NODE:
	return ((LeafNode *)currentNode)->getCategory();
      case PREDICATE_NODE:
	{
	  PredicateNode *pred=(PredicateNode *) currentNode;
	  currentNode=eval(c,*pred) ?
	    pred->getLeftChild() : pred->getRightChild();
	}
	break;
      default:
	throw TigrString("This cannot be (Classifier.C)");
      }
}



bool Classifier::eval(Case &c,PredicateNode &pred)
{
  int attribute=pred.getAttribute();
  switch(schema[attribute]->getType())
    {
    case DISCRETE:
      return c.getDiscrete(attribute)==pred.getIntValue();
    case CONTINUOUS:
      return c.getContinuous(attribute)<=
	pred.getFloatValue();
    }
}



