/****************************************************************
 DecisionTree.C
 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "DecisionTree.H"
#include <iostream>
#include <fstream>
#include "tigr++/TigrStack.H"


DecisionTree::DecisionTree(TreeNode *root)
  : root(root)
{
  // ctor
}



TreeNode *DecisionTree::getRoot()
{
  return root;
}



void DecisionTree::save(TigrString filename)
{
  ofstream os(filename.c_str());
  if(!os.good()) throw string("Cannot create file ")+filename;
  dfs(root,os);
}



void DecisionTree::dfs(TreeNode *node,ostream &os)
{
  switch(node->getType())
    {
    case LEAF_NODE:
      {
	os << int(LEAF_NODE) << ' ';

	LeafNode *leaf=(LeafNode *) node;
	os << leaf->getCategory() << endl;
      }
      break;
    case PREDICATE_NODE:
      {
	PredicateNode *pred=(PredicateNode *) node;
	
	dfs(pred->getLeftChild(),os);
	dfs(pred->getRightChild(),os);

	os << int(PREDICATE_NODE) << ' ';
	os << pred->getAttribute() << ' '
	   << pred->getFloatValue() << ' '
	   << pred->getOriginalH() << ' '
	   << pred->getMostPopularCategory()
	   << endl;
      }
      break;
    case TEMP_NODE:
      throw string("this should not happen!  (DecisionTree::dfs())");
    }
}



void DecisionTree::load(TigrString filename)
{
  ifstream is(filename.c_str());
  if(!is.good()) throw TigrString("Cannot open file ")+filename;

  TigrStack<TreeNode*> S;
  while(!is.eof())
    {
      int nodeType;
      is >> nodeType;
      if(is.eof()) break;

      switch((NodeType) nodeType)
	{
	case LEAF_NODE:
	  {
	    int category;
	    is >> category;
	    S.push(new LeafNode(category));
	  }
	  break;
	case PREDICATE_NODE:
	  {
	    int attribute, mostPopularCategory;
	    float value, originalH;
	    is >> attribute >> value >> originalH >> mostPopularCategory;
	    PredicateNode *pred=
	      new PredicateNode(attribute,value,0.0,
				mostPopularCategory,originalH);
	    pred->attachRightChild(S.top());
	    S.pop();
	    pred->attachLeftChild(S.top());
	    S.pop();
	    S.push(pred);
	  }
	  break;
	}
    }

  root=S.top();
}



void DecisionTree::getEntropies(TigrVector<float> &entropies)
{
  dfs(root,entropies);
}



void DecisionTree::dfs(TreeNode *node,TigrVector<float> &entropies)
{
  if(node->getType()==PREDICATE_NODE)
    {
      PredicateNode *pred=(PredicateNode *) node;
      
      dfs(pred->getLeftChild(),entropies);
      dfs(pred->getRightChild(),entropies);

      entropies.push_back(pred->getOriginalH());
    }
}


