/****************************************************************
 TreeNode.H
 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#ifndef INCL_TreeNode_H
#define INCL_TreeNode_H

#include "../Distribution.H"
#include "tigr++/TigrBitSet.H"

enum NodeType
{
  LEAF_NODE,
  PREDICATE_NODE,
  TEMP_NODE
};

class TreeNode
{
  NodeType type;
public:
  TreeNode(NodeType);
  virtual ~TreeNode() {}
  NodeType getType();
};

class LeafNode : public TreeNode
{
  int category;
public:
  LeafNode(int category);
  ~LeafNode() {}
  int getCategory();
};

class PredicateNode : public TreeNode
{
  int attributeNum;
  float value;
  TreeNode *leftChild, *rightChild;
  float deltaH, originalH;
  int mostPopularCategory;
public:
  PredicateNode(int attributeNum,float value,float deltaH,
		int mostPopularCategory,float originalH);
  PredicateNode(int attributeNum,int value);
  ~PredicateNode() {delete leftChild;delete rightChild;}
  void attachLeftChild(TreeNode *);
  void attachRightChild(TreeNode *);
  TreeNode *getLeftChild();
  TreeNode *getRightChild();
  int getAttribute();
  float getFloatValue();
  int getIntValue();
  float getDeltaH();
  float getOriginalH();
  int getMostPopularCategory();
};

class TempNode : public TreeNode
{
  TigrBitSet *set;//deleted in dtor
  Distribution *distribution;//deleted in dtor
  TreeNode *parent;
public:
  TempNode(TigrBitSet *,Distribution *,TreeNode *parent);
  virtual ~TempNode();
  TreeNode *getParent();
  TigrBitSet *getSet();
  Distribution *getDistribution();
};

#endif
