/****************************************************************
 CrosserOver.C
 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "CrosserOver.H"
#include <iostream>
#include "tigr++/TigrRandom.H"
#include "Tableau.H"
using namespace std;


CrosserOver::CrosserOver()
{
  // ctor
}



void CrosserOver::crossover(const Individual *mother,
			    const Individual *father,
			    Individual *&daughter,Individual *&son)
{
  // Select a tree
  const int treeNum=RandomNumber(mother->getNumTrees());

  // We actualy clone the mother and father and then cross over
  // the children, leaving mom & dad intact
  son=father->clone();
  daughter=mother->clone();


  // Select a crossover point in each individual
  CrossoverPoint sonPoint=selectCrossoverPoint(*son,treeNum);
  CrossoverPoint daughterPoint=selectCrossoverPoint(*daughter,treeNum);

  // Perform crossover
  sonPoint.performCrossover(&daughterPoint);

  // Enforce height limit
  int maxHeight=Tableau::getGlobal().getMaxHeight();
  if(maxHeight>0)
    {
      enforceHeightLimit(son,maxHeight,father);
      enforceHeightLimit(daughter,maxHeight,mother);
    }
}



void CrosserOver::enforceHeightLimit(Individual *&child,int maxHeight,
				     const Individual *parent)
{
  int childHeight=child->getTreeHeight();
  if(childHeight<=maxHeight) return;

  delete child;
  child=parent->clone();
}



CrossoverPoint CrosserOver::selectCrossoverPoint(Individual &individual,
						 int treeNum)
{
  TreeNode *root=individual.getTree(treeNum);

  // Get all possible crossover points
  vector<CrossoverPoint> &points=*crossPtCollector.collectPoints(root);

  // Randomly pick a crossover point
  int pointNum=RandomNumber(points.size());
  CrossoverPoint point=points[pointNum];
  delete &points;

  // Return the selected point
  return point;
}



