/****************************************************************
 Tableau.C
 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "Tableau.H"
#include <iostream>
#include <stdlib.h>
using namespace std;


Tableau *Tableau::global;


Tableau::Tableau(TigrConfigFile &config)
{
  // ctor

  G=config.getIntOrDie("max-generations");
  M=config.getIntOrDie("population-size");
  logFilename=config.lookupOrDie("log-file");
  Pc=config.getFloatOrDie("crossover");
  Pm=config.getFloatOrDie("point-mutation");
  subtreeMutation=config.getFloatOrDie("subtree-mutation");
  immigration=config.getFloatOrDie("immigration");
  if(Pc+Pm<0 || Pc+Pm>1.0) 
    throw TigrString("proportions of crossover & mutation must sum to a\n"
		 "number between 0 and 1");
  minConst=config.getFloatOrDie("min-const");
  maxConst=config.getFloatOrDie("max-const");
  initialHeight=config.getIntOrDie("initial-tree-height");
  maxHeight=config.getIntOrDie("max-tree-height");
  seed=config.getIntOrDie("seed");
  maxCall=config.getIntOrDie("max-adf-call");
  adfArities=config.lookupOrDie("adf-arities");
  entryArities=config.lookupOrDie("entry-point-arities");
  nonterminals=config.lookupOrDie("nonterminals");
  terminals=config.lookupOrDie("terminals");
  resultBranch=config.getIntOrDie("result-producing-branch");
  trainPortion=config.getFloatOrDie("percent-training-set");
  tournamentSize=config.getIntOrDie("tournament-size");
  tournament=config.getIntOrDie("tournament-selection") ? 
    true : false;
  cloning=config.getFloatOrDie("cloning");
}



void Tableau::processConfig(TigrConfigFile &config)
{
  global=new Tableau(config);
}



Tableau &Tableau::getGlobal()
{
  return *global;
}



