/****************************************************************
 Tableau.H
 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#ifndef INCL_Tableau_H
#define INCL_Tableau_H

#include "tigr++/TigrConfigFile.H"

class Tableau
{
  static Tableau *global;
  int G; // max # generations
  int M; // population size
  float Pc; // proportion of population produced using crossover
  float Pm; // proportion of population produced using mutation
  float immigration; // amount of pop. produced using immigration
  float cloning; // amount of pop. produced using cloning
  float subtreeMutation; // how much of pop. comes from subtree mutation
  float minConst; // min value of a constant node
  float maxConst; // max value of a constant node
  int initialHeight; // max tree height of intial population
  int maxHeight; // max height for any tree, any population
  int seed; // seed for randomizer, or 0 if none
  int maxCall; // max # calls to ADF's (Automatically-Defined Functions)
  TigrString logFilename; // filename of log file
  TigrString adfArities; // comma-separated list of ADF arities
  TigrString entryArities; // comma-sep. list of entry-point arities
  TigrString nonterminals; // comma-sep. list of nonterminals (operators)
  TigrString terminals; // comma-sep. list of terminals (leaves)
  int resultBranch; // which entry-point actually computes the answer
  float trainPortion; // how much of the training set each individual
                      // is exercised on during training
  bool tournament; // whether to use tournament selection rather
                   // than fitness-proportionate selection
  int tournamentSize;

  Tableau(TigrConfigFile &);
public:
  static void processConfig(TigrConfigFile &);
  static Tableau &getGlobal();
  int getG() {return G;}
  int getM() {return M;}
  float getPc() {return Pc;}
  float getPm() {return Pm;}
  float getSubtreeMutation() {return subtreeMutation;}
  float getMinConst() {return minConst;}
  float getMaxConst() {return maxConst;}
  int getInitialHeight() {return initialHeight;}
  int getMaxHeight() {return maxHeight;}
  int getSeed() {return seed;}
  int getMaxCall() {return maxCall;}
  TigrString getAdfArities() {return adfArities;}
  TigrString getEntryArities() {return entryArities;}
  TigrString getNonterminals() {return nonterminals;}
  TigrString getTerminals() {return terminals;}
  int getResultBranch() {return resultBranch;}
  float getTrainPortion() {return trainPortion;}
  bool useTournament() {return tournament;}
  int getTournamentSize() {return tournamentSize;}
  float getImmigration() {return immigration;}
  float getCloning() {return cloning;}
};


#endif
