/****************************************************************
 NamesFile.C
 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "NamesFile.H"
#include <iostream>
#include <fstream>
#include "tigr++/TigrStrTokenizer.H"


NamesFile::NamesFile()
{
  // ctor
}



Schema *NamesFile::processFile(const TigrString &filename)
{
  ifstream is(filename.c_str());
  if(!is.good())
    throw TigrString("Cannot open names file: ")+filename;
  
  // Process first line (number of categories)
  TigrString line;
  getline(is,line);
  if(line.length()==0)
    throw TigrString("Syntax error in names file");
  TigrStrTokenizer tokenizer(line.c_str()," \t\n\r.:");
  TigrString numString=tokenizer.nextToken();
  int numCategories=numString.asInt();
  if(numCategories<=0)
    throw TigrString("bad number of categories in names file");
  TigrString cat=tokenizer.nextToken();
  if(!(cat=="categories"))
    throw TigrString("Syntax error in names file - ")+
      "expecting keyword \"categories\"";

  Schema *schema=new Schema(numCategories);

  // Process subsequent lines
  while(!is.eof())
    {
      getline(is,line);
      if(line.length()==0) break;

      TigrStrTokenizer tokenizer(line.c_str()," \t\n\r.:");
      while(tokenizer.hasMoreTokens())
	{
	  // Read the next line
	  TigrString attribute=tokenizer.nextToken();
	  TigrString typeString=tokenizer.nextToken();
	  
	  // Allocate a type descriptor
	  TypeDescriptor *descriptor=NULL;
	  if(typeString=="discrete")
	    descriptor=new Discrete(0,attribute);
	  else if(typeString=="continuous")
	    descriptor=new Continuous(attribute);
	  else
	    throw TigrString("Illegal attribute type in names file");
	  
	  // Add the type descriptor to the schema
	  schema->add(descriptor);

	  // Store the name
	  names.push_back(attribute);
	}
    }

  return schema;
}



TigrString &NamesFile::getName(int i)
{
  return names[i];
}



