/****************************************************************
 Schema.C
 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "Schema.H"
#include "tigr++/TigrArray1D.H"
#include "TrainingSet.H"


Schema::Schema(int num)
  : numCategories(num)
{
  // ctor
}



void Schema::add(TypeDescriptor *d)
{
  v.push_back(d);
}



int Schema::numAttributes()
{
  return v.size();
}



TypeDescriptor *Schema::operator[](int i)
{
  return v[i];
}



int Schema::getNumCategories()
{
  return numCategories;
}



void Schema::removeAttribute(int i)
{
  delete v[i];
  v.cut(i);
}



void Schema::printOn(ostream &os) const
{
  os << numCategories << " categories" << endl;

  int i, n=v.size();
  for(i=0 ; i<n ; ++i)
    os << *v[i] << endl;
}



ostream &operator<<(ostream &os,const Schema &schema)
{
  schema.printOn(os);
  return os;
}



TigrMap<int,int> *Schema::countDiscrete(TrainingSet &set)
{
  // First, get indices of discrete attributes
  TigrVector<int> indices;
  int i, numAttr=numAttributes();
  for(i=0 ; i<numAttr ; ++i)
    if(v[i]->getType()==DISCRETE) 
      indices.push_back(i);
  int numDiscreteAttr=indices.size();

  // Iterate through the cases, noting each unique
  // value of each discrete attribute
  TigrMap<int,int> *maps=new TigrMap<int,int>[numDiscreteAttr];
  int numCases=set.numCases();
  TigrArray1D<int> nextLevel(numDiscreteAttr);
  nextLevel.setAllTo(0);
  for(i=0 ; i<numCases ; ++i)
    {
      TrainingCase &c=set.getCase(i);
      int j;
      for(j=0 ; j<numDiscreteAttr ; ++j)
	{
	  int attr=indices[j];
	  int level=c.getDiscrete(attr);
	  if(maps[j].find(level)==maps[j].end())
	    maps[j][level]=nextLevel[j]++;
	}
    }

  // Store counts in individual TypeDescriptors
  for(i=0 ; i<numDiscreteAttr ; ++i)
    {
      int index=indices[i];
      Discrete &discrete=(Discrete&)*v[index];
      discrete.setN(maps[i].size());

      cout << "setting count " << index << " to " << maps[i].size()//###
	   << endl; // ###
    }

  return maps;
}


