/****************************************************************
 TrainingSet.C
 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "TrainingSet.H"
#include <iostream>
#include <fstream>
#include <math.h>
#include "tigr++/TigrStrTokenizer.H"
#include <stdlib.h>


TrainingSet::TrainingSet(Schema &schema,TigrString filename)
  : schema(schema)
{
  // ctor

  load(filename);
}



TrainingSet::TrainingSet(Schema &schema)
  : schema(schema)
{
  // ctor
}



void TrainingSet::load(TigrString &filename)
{
  int numAttributes=schema.numAttributes();

  ifstream is(filename.c_str());
  if(!is.good()) throw TigrString("can't open file \"")+filename+"\"";
  while(!is.eof())
    {
      // Load line containing next training case
      TigrString line;
      getline(is,line);
      if(line.length()==0 || is.eof()) break;

      // Parse attribute values
      TrainingCase *newCase=new TrainingCase(numAttributes);
      TigrStrTokenizer tokenizer(line," \t\n\r");
      for(int i=0 ; i<numAttributes ; ++i)
	{
	  if(!tokenizer.hasMoreTokens())
	    throw TigrString("Incomplete case");
	  const char *token=tokenizer.nextToken();

	  switch(schema[i]->getType())
	    {
	    case DISCRETE:
	      newCase->setDiscrete(i,atoi(token));
	      break;
	    case CONTINUOUS:
	      newCase->setContinuous(i,atof(token));
	      break;
	    }
	}

      // Parse the category
      if(!tokenizer.hasMoreTokens())
	throw TigrString("Incomplete case");
      const char *token=tokenizer.nextToken();
      newCase->setCategory(atoi(token));
      if(tokenizer.hasMoreTokens())
	throw TigrString("Case has too many fields");

      // Add training case to theVector
      theVector.push_back(newCase);
    }
}



TrainingCase &TrainingSet::getCase(int i)
{
  return *theVector[i];
}



int TrainingSet::numCases()
{
  return theVector.size();
}



Schema &TrainingSet::getSchema()
{
  return schema;
}



void TrainingSet::addCase(TrainingCase *newCase)
{
  theVector.push_back(newCase);
}



TrainingSet *TrainingSet::getCategory(int category)
{
  TrainingSet &newSet=*new TrainingSet(schema);

  int i, n=numCases();
  for(i=0 ; i<n ; ++i)
    {
      TrainingCase &thisCase=getCase(i);
      if(thisCase.getCategory()==category)
	newSet.addCase(&thisCase);
    }

  return &newSet;
}



void TrainingSet::deleteCases()
{
  int n=theVector.size();
  for(int i=0 ; i<n ; ++i)
    delete theVector[i];
}


