/****************************************************************
 NeuralNetwork.H
 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#ifndef INCL_NeuralNetwork_H
#define INCL_NeuralNetwork_H

#include "tigr++/TigrStacktrace.H"
#include "tigr++/TigrVector.H"
#include "tigr++/TigrArray1D.H"
#include <iostream>
using namespace std;

class Neuron;

class NeuralNetwork
{
  TigrVector<Neuron*> neurons;
  TigrVector<int> inputs, outputs;
  bool validIndex(int);
  Neuron *getNeuron(int);//no index checking
  void insertInts(TigrVector<int> &,int value,int num);
public:
  NeuralNetwork();
  virtual ~NeuralNetwork();
  void addNeuron(Neuron *);
  void addInputNeuron(Neuron *);
  void addOutputNeuron(Neuron *);
  void connect(int from,int to,float weight);
  Neuron &operator[](int);// throws exception
  void setInput(int whichNode,float value);
  void propagate();
  float getOutput(int whichNode);
  int getNumNeurons() const;
  TigrVector<int> &getInputIndices();
  TigrVector<int> &getOutputIndices();
  int getNumInputs();
  int getNumOutputs();
  TigrVector<int> *topologicalSort();//only for feedforward nets!
  void dump(ostream &);
  void saveSelf(ostream &);
  void loadSelf(istream &);
  int countSynapses();
  void copySynapsesOut(TigrFloatArray1D &);
  void copySynapsesIn(TigrFloatArray1D &);
};

#endif
