#include <iostream>
#include "TigrVector.H"
#include "TigrFile.H"
#include "TigrChi2Table.H"
using namespace std;



TigrChi2Table::TigrChi2Table(TigrString filename)
  : DFs(35), alphas(12), table(12,35)
{
   
  int row=0;
  TigrFile file(filename);
   
  TigrString line=file.readLine();
  TigrVector<TigrString> &fields=*line.getFields();
  int n=fields.size();
  for(int i=1 ; i<n ; ++i)
    alphas[i-1]=fields[i].asFloat();
  delete &fields;

   
  while(!file.eof())
    {
      TigrString line=file.readLine();
      if(file.eof()) break;
      TigrVector<TigrString> *fieldsPtr=line.getFields();
      if(!fieldsPtr) continue;
      TigrVector<TigrString> &fields=*fieldsPtr;
      int n=fields.size();
      int df=fields[0].asInt();
      DFs[row]=df;
      for(int i=1 ; i<n ; ++i)
	{
	  float chi=fields[i].asFloat();
	  table[i-1][row]=chi;
	}
      delete fieldsPtr;
      ++row;
    }
}



float TigrChi2Table::lookupChi(int df,float alpha)
{
  int alphaIndex=alphaToIndex(alpha);
  int dfIndex=dfToIndex(df);

  if(df<DFs[dfIndex])
    {
      float y1=table[alphaIndex][dfIndex-1];
      float y2=table[alphaIndex][dfIndex];
      int x1=DFs[dfIndex-1];
      int x2=DFs[dfIndex];
      float ratio=float(df-x1)/(x2-x1);
      float chi=y1+ratio*(y2-y1);
      return chi;
    }
  else if(df>DFs[dfIndex] && dfIndex<DFs.size())
    {
      float y1=table[alphaIndex][dfIndex];
      float y2=table[alphaIndex][dfIndex+1];
      int x1=DFs[dfIndex];
      int x2=DFs[dfIndex+1];
      float ratio=float(df-x1)/(x2-x1);
      float chi=y1+ratio*(y2-y1);
      return chi;
    }
  if(dfIndex>=DFs.size()) dfIndex=DFs.size()-1;
  
  return table[alphaIndex][dfIndex];
}



float TigrChi2Table::lookupP(int df,float chi)
{
  int dfIndex=dfToIndex(df);
  unsigned begin=0, end=alphas.size();
  while(begin<end)
    {
      unsigned mid=unsigned((begin+end)/2);
      float midChi=table[mid][dfIndex];
      if(chi>midChi) begin=mid+1;
      else end=mid;
    }
  if(begin>=alphas.size()) begin=alphas.size()-1;
  if(chi<table[begin][dfIndex]) 
    {
      if(begin==0) return alphas[0];
      float x1=table[begin-1][dfIndex];
      float x2=table[begin][dfIndex];
      float y1=alphas[begin-1]; 
      float y2=alphas[begin];
      float ratio=(chi-x1)/float(x2-x1);
      return y1-ratio*(y1-y2);
    }
  else if(chi>table[begin][dfIndex] && begin<alphas.size()-1)
    {
      float x1=table[begin][dfIndex];
      float x2=table[begin+1][dfIndex];
      float y1=alphas[begin];
      float y2=alphas[begin+1];
      float ratio=(chi-x1)/float(x2-x1);
      return y1-ratio*(y1-y2);
    }

  return alphas[begin];
}



unsigned TigrChi2Table::alphaToIndex(float alpha)
{
  unsigned begin=0, end=alphas.size();
  while(begin<end)
    {
      unsigned mid=unsigned((begin+end)/2);
      float midAlpha=alphas[mid];
      if(alpha<midAlpha) begin=mid+1;
      else end=mid;
    }
  if(alphas[begin]!=alpha) 
    throw TigrString("Bad alpha in TigrChi2Table::lookupChi(")+
      alpha+")";
  return begin;
}



unsigned TigrChi2Table::dfToIndex(int df)
{
  unsigned begin=0, end=DFs.size();
  while(begin<end)
    {
      unsigned mid=unsigned((begin+end)/2);
      int midElem=DFs[mid];
      if(df>midElem) begin=mid+1;
      else end=mid;
    }
  return begin;
}
